"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AliasSupportPlugin = void 0;
const tslib_1 = require("tslib");
const utils_1 = require("@morjs/utils");
const path_1 = tslib_1.__importDefault(require("path"));
const constants_1 = require("../constants");
const getAliasIdentifyRegex = (alias) => new RegExp(Object.keys(alias).reduce((a, b) => (a === '' ? b : a + '|' + b), ''));
const IMPORT_MODULE_NAME_REGEX = /import(?:['"\s]*([\w*${}\s,]+)from\s*)?['"\s]['"\s](.*[@\w_-]+)['"\s].*/g;
const REQUIRE_MODULE_NAME_REGEX = /require\s*\(([`'"])\s*([^\s]*)\s*([`'"])\)/g;
/**
 * 实现 transform 模式下 alias 支持
 * alias 配置参见：http://mor.eleme.io/guides/basic/config#alias---%E5%88%AB%E5%90%8D%E9%85%8D%E7%BD%AE
 *
 * 注意：alias 配置会对 EntryFileType.script 和 EntryFileType.sjs 文件类型生效
 */
class AliasSupportPlugin {
    constructor() {
        this.name = 'AliasSupportPlugin';
    }
    apply(runner) {
        runner.hooks.webpackWrapper.tap(this.name, (webpackWrapper) => {
            this.webpackWrapper = webpackWrapper;
        });
        runner.hooks.entryBuilder.tap(this.name, (entryBuilder) => {
            this.entryBuilder = entryBuilder;
        });
        const matchedModuleNamesInfo = {};
        runner.hooks.userConfigValidated.tap({
            name: this.name
        }, (userConfig) => {
            if (userConfig.compileMode !== constants_1.CompileModes.transform)
                return;
            if (!userConfig.alias)
                return;
            runner.hooks.preprocessorParser.tapPromise({
                name: this.name
            }, async (fileContent, context, options) => {
                const fileInfo = options.fileInfo;
                /*
                 * 仅处理 EntryFileType.script 和 EntryFileType.sjs 文件
                 */
                if ([utils_1.EntryFileType.script, utils_1.EntryFileType.sjs].includes(fileInfo.entryFileType)) {
                    await this.handleAliasPrepocesser(userConfig, fileContent, options, matchedModuleNamesInfo);
                }
                return fileContent;
            });
            runner.hooks.scriptParser.tap({
                name: this.name
            }, (transformers, options) => {
                this.handleAliasTransformer(transformers, options, matchedModuleNamesInfo);
                return transformers;
            });
            runner.hooks.sjsParser.tap({
                name: this.name
            }, (transformers, options) => {
                this.handleAliasTransformer(transformers, options, matchedModuleNamesInfo);
                return transformers;
            });
        });
    }
    async handleAliasPrepocesser(userConfig, fileContent, options, matchedModuleNamesInfo) {
        /**
         * 减少处理损耗，判断文件内容内是否包含 alias 对象 key
         */
        const ALIAS_IDENTIFY_REGEX = getAliasIdentifyRegex(userConfig.alias);
        if (!ALIAS_IDENTIFY_REGEX.test(fileContent))
            return;
        /**
         * 避免注释文件影响，移除 comment 注释
         */
        const content = (await utils_1.esbuild.transform(fileContent, {
            loader: 'ts',
            target: 'esnext',
            legalComments: 'none'
        })).code;
        const resolver = this.getEnhanceResolver({});
        const filePath = options.fileInfo.path;
        /**
         * 获取所有匹配标识符
         */
        const matches = [
            ...content.matchAll(IMPORT_MODULE_NAME_REGEX),
            ...content.matchAll(REQUIRE_MODULE_NAME_REGEX)
        ];
        const moduleNamesList = await Promise.all([...matches]
            .filter((matchItem) => {
            return Object.keys(userConfig.alias).some((item) => {
                return new RegExp(item).test(matchItem[2]);
            });
        })
            .map(async (matchItem) => {
            const requestPath = matchItem[2];
            const { realPath } = await new Promise((resolve, reject) => {
                resolver.resolve({}, filePath, requestPath, {}, (err, realPath) => {
                    if (err)
                        return reject(err);
                    if (realPath) {
                        resolve({ realPath: realPath });
                    }
                    else {
                        reject(new Error(`未在 node_modules 中找到文件: ${requestPath}`));
                    }
                });
            });
            /**
             * 需要主动移除一个层级，因为是通过 filePath 而非文件夹路径取的相对路径
             */
            let realImportPath = path_1.default.relative(filePath, realPath).slice(3);
            realImportPath = (0, utils_1.slash)(realImportPath.startsWith('.')
                ? realImportPath
                : './' + realImportPath);
            /**
             * 替换引用文件后缀, replaceExtAsExpected 暂不处理 ts 文件类型此处兼容处理下
             * 如：ts 后缀需要替换为编译后的 js
             */
            realImportPath = this.entryBuilder.replaceExtAsExpected(realImportPath.replace(/\.ts/, '.js'));
            return { requestPath, realImportPath };
        }));
        /**
         * 便于在后续 hook 中替换，此处存储匹配信息
         * 优化存储空间，仅存储有内容的 filePath
         */
        if (moduleNamesList.length === 0)
            return;
        matchedModuleNamesInfo[filePath] = {};
        moduleNamesList.forEach((item) => {
            matchedModuleNamesInfo[filePath][item.requestPath] = item.realImportPath;
        });
    }
    handleAliasTransformer(transformers, options, names) {
        const filePath = options.fileInfo.path;
        if (!names[filePath]) {
            return transformers;
        }
        transformers.before.push((0, utils_1.tsTransformerFactory)(function (node, context) {
            var _a, _b;
            if (!node)
                return node;
            const factory = context.factory;
            /**
             * 处理 import 节点
             */
            if (utils_1.typescript.isImportDeclaration(node) &&
                utils_1.typescript.isStringLiteral(node.moduleSpecifier) &&
                names[filePath][node.moduleSpecifier.text]) {
                return factory.updateImportDeclaration(node, node.decorators, node.modifiers, node.importClause, factory.createStringLiteral(names[filePath][node.moduleSpecifier.text]));
            }
            /**
             * 处理 require 节点
             */
            if (utils_1.typescript.isCallExpression(node) &&
                utils_1.typescript.isIdentifier(node.expression) &&
                ((_a = node.expression) === null || _a === void 0 ? void 0 : _a.escapedText) === 'require' &&
                utils_1.typescript.isStringLiteral((_b = node.arguments) === null || _b === void 0 ? void 0 : _b[0]) &&
                names[filePath][node.arguments[0].text]) {
                return factory.createCallExpression(factory.createIdentifier('require'), undefined, [
                    factory.createStringLiteral(names[filePath][node.arguments[0].text])
                ]);
            }
            return node;
        }));
        return transformers;
    }
    /**
     * 获取 webpack normal resolver
     * @param options - ResolverOptions 用于传递给 resolver
     */
    getEnhanceResolver(options) {
        return this.webpackWrapper.compiler.resolverFactory.get('normal', options);
    }
}
exports.AliasSupportPlugin = AliasSupportPlugin;
//# sourceMappingURL=aliasSupportPlugin.js.map